
#include "derivednodes.h"
#include "regalloc.h"

extern int tabCount;
int labelNum = 0;
extern RegisterAllocator *regAlloc;

///////////////////////////////////////////////////////////
// Compound Statement Node Implementation
///////////////////////////////////////////////////////////
COMPOUND_STMT_Node::COMPOUND_STMT_Node():ASTNode()
{
    decls=NULL;
}

COMPOUND_STMT_Node::~COMPOUND_STMT_Node()
{
}

void COMPOUND_STMT_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Compound_statement>\n", tab.c_str());

    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;

    fprintf(fout, "%s</Compound_statement>\n", tab.c_str());

}

bool COMPOUND_STMT_Node::addDecl(ASTNode *leaf)
{
    if ( leaf != NULL )
    {
        decls=leaf;
        return true;
    }
    else
    {
        printf("Warning: Adding NULL Declaration leaf to COMPOUND AST!!\n");
        return false;
    }
}

ASTNode* COMPOUND_STMT_Node::getDecl()
{
    return decls;
}

///////////////////////////////////////////////////////////
// Labeled Statement Node Implementation
///////////////////////////////////////////////////////////
LABELED_STMT_Node::LABELED_STMT_Node():ASTNode()
{
    production="";
}

LABELED_STMT_Node::~LABELED_STMT_Node()
{
}

void LABELED_STMT_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Labeled_stmt>\n", tab.c_str());

    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }

    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;

    fprintf(fout, "%s</Labeled_stmt>\n", tab.c_str());

}

///////////////////////////////////////////////////////////
// Selection Statement Node Implementation
///////////////////////////////////////////////////////////
SELECTION_STMT_Node::SELECTION_STMT_Node():ASTNode()
{
}

SELECTION_STMT_Node::~SELECTION_STMT_Node()
{
}


void SELECTION_STMT_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Selection_stmt>\n", tab.c_str());

    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }

    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;

    fprintf(fout, "%s</Selection_stmt>\n", tab.c_str());
}

void SELECTION_STMT_Node::prologue(FILE *fout)
{
    fprintf(fout,"\n\t# Selection Statement prologue");
    // get the return register from expr node
    //jump if return_reg is 0
    if (getTypeSpecifier() == "NO_ELSE")
        fprintf(fout,"\n\tbeqz\t%s, %s\n", regAlloc->getLastRegister().c_str(), getEndLabel().c_str());
    if (getTypeSpecifier() == "ELSE")
        fprintf(fout,"\n\tbeqz\t%s, %s\n", regAlloc->getLastRegister().c_str(), getStartLabel().c_str());
    //generate label
}
void SELECTION_STMT_Node::epilogue(FILE *fout)
{
    fprintf(fout,"\n\t# Selection Statement epilogue");
    fprintf(fout,"\n%s: \n",getEndLabel().c_str());
    //generate label
}


///////////////////////////////////////////////////////////
// Statement Node Implementation
///////////////////////////////////////////////////////////
STATEMENT_Node::STATEMENT_Node():ASTNode()
{
}

STATEMENT_Node::~STATEMENT_Node()
{
}


void STATEMENT_Node::prologue(FILE *fout)
{
    fprintf(fout,"\n\t# Statement Prologue");

}
void STATEMENT_Node::epilogue(FILE *fout)
{
    fprintf(fout,"\n\t# Statement Epilogue");

}

void STATEMENT_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Statement>\n", tab.c_str());
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Statement>\n", tab.c_str());
}


///////////////////////////////////////////////////////////
// Iteration Statement Node Implementation
///////////////////////////////////////////////////////////
ITERATION_STMT_Node::ITERATION_STMT_Node():ASTNode()
{
    loop = "While";
    EXPR_A = NULL;
    EXPR_B = NULL;
    EXPR_C = NULL;
    STMT = NULL;
}

ITERATION_STMT_Node::~ITERATION_STMT_Node()
{
}

void ITERATION_STMT_Node::prologue(FILE *fout)
{
	//generate a label to jump to the beginning of the loop
	//generate a jump to jump to the end of loop
	// 	beqz 	$t0, end_for
    fprintf(fout,"\n\t# Iteration Prologue");
    fprintf(fout,"\n%s:\n",getStartLabel().c_str());

}
void ITERATION_STMT_Node::epilogue(FILE *fout)
{
	//just generate a label to jump to
    fprintf(fout,"\n\t# Iteration Epilogue");
    fprintf(fout,"\n\tj\t%s\n",getStartLabel().c_str());
    fprintf(fout,"\n%s:\n",getEndLabel().c_str());

}




void ITERATION_STMT_Node::setLoopType(string _loop)
{
    loop = _loop;
}

string ITERATION_STMT_Node::getLoopType()
{
    return loop;
}

void ITERATION_STMT_Node::addBranch(ASTNode *_branch, string _branchType)
{
    if ( _branchType == "EXPR_A" )
        EXPR_A = _branch;
    else if ( _branchType == "EXPR_B" )
        EXPR_B = _branch;
    else if ( _branchType == "EXPR_C" )
        EXPR_C = _branch;
    else if ( _branchType == "STMT" )
        STMT = _branch;
}

void ITERATION_STMT_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Iteration_statement type=\"%s\" >\n", 
            tab.c_str(), loop.c_str());

    ++tabCount;

    if ( EXPR_A != NULL )
        EXPR_A->DumpOut(fout);

    if ( EXPR_B != NULL )
        EXPR_B->DumpOut(fout);

    if ( EXPR_C != NULL )
        EXPR_C->DumpOut(fout);

    --tabCount;

    fprintf(fout, "%s</Iteration_statement>\n", tab.c_str());

}

///////////////////////////////////////////////////////////
// Expression Node Implementation
///////////////////////////////////////////////////////////
EXPR_Node::EXPR_Node():ASTNode()
{
}

EXPR_Node::~EXPR_Node()
{
}

void EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Expr>\n", tab.c_str());
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Expr>\n", tab.c_str());

}

///////////////////////////////////////////////////////////
// Assignment Expression Node Implementation
///////////////////////////////////////////////////////////
ASSIGNMENT_EXPR_Node::ASSIGNMENT_EXPR_Node():ASTNode()
{
    production="";
}

ASSIGNMENT_EXPR_Node::~ASSIGNMENT_EXPR_Node()
{
}

void ASSIGNMENT_EXPR_Node::DumpOut(FILE *fout)
{

    string tab = setIndentTab();
    string tab2 = setIndentTab();
    fprintf(fout, "%s<Assignment>\n", tab.c_str());

    if (production.size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), production.c_str());
    }

    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Assignment>\n", tab.c_str());


}

void ASSIGNMENT_EXPR_Node::setProduction(string str)
{
    production = str;
}

///////////////////////////////////////////////////////////
// Primary expression Node Implementation
///////////////////////////////////////////////////////////
PRIMARY_EXPR_Node::PRIMARY_EXPR_Node():ASTNode()
{
}

PRIMARY_EXPR_Node::~PRIMARY_EXPR_Node()
{
}

void PRIMARY_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Primary_expr>\n", tab.c_str());
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Primary_expr>\n", tab.c_str());

}

///////////////////////////////////////////////////////////
// Argument expression list Node Implementation
///////////////////////////////////////////////////////////
ARGUMENT_EXPR_LIST_Node::ARGUMENT_EXPR_LIST_Node():ASTNode()
{
}

ARGUMENT_EXPR_LIST_Node::~ARGUMENT_EXPR_LIST_Node()
{
}

void ARGUMENT_EXPR_LIST_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Argument_expression_list>\n", tab.c_str());
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Argument_expression_list>\n", tab.c_str());
}

///////////////////////////////////////////////////////////
// Conditional expression Node Implementation
///////////////////////////////////////////////////////////
CONDITIONAL_EXPR_Node::CONDITIONAL_EXPR_Node():ASTNode()
{
}

CONDITIONAL_EXPR_Node::~CONDITIONAL_EXPR_Node()
{
}

void CONDITIONAL_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Conditional_expr>\n", tab.c_str());
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Conditional_expr>\n", tab.c_str());


}

///////////////////////////////////////////////////////////
// Logical or expression Node Implementation
///////////////////////////////////////////////////////////
LOGICAL_OR_EXPR_Node::LOGICAL_OR_EXPR_Node():ASTNode()
{
}

LOGICAL_OR_EXPR_Node::~LOGICAL_OR_EXPR_Node()
{
}

void LOGICAL_OR_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Logical_or_expr>\n", tab.c_str());

    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }

    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Logical_or_expr>\n", tab.c_str());


}

///////////////////////////////////////////////////////////
// Logical and expression Node Implementation
///////////////////////////////////////////////////////////
LOGICAL_AND_EXPR_Node::LOGICAL_AND_EXPR_Node():ASTNode()
{
}

LOGICAL_AND_EXPR_Node::~LOGICAL_AND_EXPR_Node()
{
}

void LOGICAL_AND_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Logical_and_expr>\n", tab.c_str());
    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Logical_and_expr>\n", tab.c_str());


}


///////////////////////////////////////////////////////////
// Inclusive or expression Node Implementation
///////////////////////////////////////////////////////////
INCLUSIVE_OR_EXPR_Node::INCLUSIVE_OR_EXPR_Node():ASTNode()
{
}

INCLUSIVE_OR_EXPR_Node::~INCLUSIVE_OR_EXPR_Node()
{
}

void INCLUSIVE_OR_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Inclusive_or_expr>\n", tab.c_str());
    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }

    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Inclusive_or_expr>\n", tab.c_str());


}

///////////////////////////////////////////////////////////
// Exclusive or expression list Node Implementation
///////////////////////////////////////////////////////////
EXCLUSIVE_OR_EXPR_Node::EXCLUSIVE_OR_EXPR_Node():ASTNode()
{
}

EXCLUSIVE_OR_EXPR_Node::~EXCLUSIVE_OR_EXPR_Node()
{
}

void EXCLUSIVE_OR_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Exclusive_or_expr>\n", tab.c_str());
    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Exclusive_or_expr>\n", tab.c_str());


}



///////////////////////////////////////////////////////////
// And expression Node Implementation
///////////////////////////////////////////////////////////
AND_EXPR_Node::AND_EXPR_Node():ASTNode()
{
}

AND_EXPR_Node::~AND_EXPR_Node()
{
}

void AND_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<And_expr>\n", tab.c_str());

    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }

    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</And_expr>\n", tab.c_str());


}


///////////////////////////////////////////////////////////
// Argument expression list Node Implementation
///////////////////////////////////////////////////////////
EQUALITY_EXPR_Node::EQUALITY_EXPR_Node():ASTNode()
{
    pro = 0;
    production="";
}

EQUALITY_EXPR_Node::~EQUALITY_EXPR_Node()
{
}

void EQUALITY_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Equality_expr>\n", tab.c_str());
    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Equality_expr>\n", tab.c_str());


}

void EQUALITY_EXPR_Node::setProduction(char *i)
{
    production = i;
}


///////////////////////////////////////////////////////////
// Relational expression list Node Implementation
///////////////////////////////////////////////////////////
RELATIONAL_EXPR_Node::RELATIONAL_EXPR_Node():ASTNode()
{
    production="";
}

RELATIONAL_EXPR_Node::~RELATIONAL_EXPR_Node()
{
}

void RELATIONAL_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Relational_expr>\n", tab.c_str());
    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Relational_expr>\n", tab.c_str());


}

void RELATIONAL_EXPR_Node::setProduction(char *i)
{
    production = i;
}


///////////////////////////////////////////////////////////
// Shift expression Node Implementation
///////////////////////////////////////////////////////////
SHIFT_EXPR_Node::SHIFT_EXPR_Node():ASTNode()
{
}

SHIFT_EXPR_Node::~SHIFT_EXPR_Node()
{
}

void SHIFT_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Shift_expr>\n", tab.c_str());
    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Shift_expr>\n", tab.c_str());

}


///////////////////////////////////////////////////////////
// Additive expression Node Implementation
///////////////////////////////////////////////////////////
ADDITIVE_EXPR_Node::ADDITIVE_EXPR_Node():ASTNode()
{
    production="";
}

ADDITIVE_EXPR_Node::~ADDITIVE_EXPR_Node()
{
}

void ADDITIVE_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Additive_expr>\n", tab.c_str());
    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s   <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Additive_expr>\n", tab.c_str());

}

void ADDITIVE_EXPR_Node::setProduction(char *i)
{
    production = i;
}


///////////////////////////////////////////////////////////
// Multiplicative expression list Node Implementation
///////////////////////////////////////////////////////////
MULTIPLICATIVE_EXPR_Node::MULTIPLICATIVE_EXPR_Node():ASTNode()
{
    production="";
}

MULTIPLICATIVE_EXPR_Node::~MULTIPLICATIVE_EXPR_Node()
{
}

void MULTIPLICATIVE_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Multiplicative_expr>\n", tab.c_str());
    if (getTypeSpecifier().size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), getTypeSpecifier().c_str());
    }
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Multiplicative_expr>\n", tab.c_str());


}

void MULTIPLICATIVE_EXPR_Node::setProduction(char *i)
{
    production = i;
}


///////////////////////////////////////////////////////////
// Cast expression Node Implementation
///////////////////////////////////////////////////////////
CAST_EXPR_Node::CAST_EXPR_Node():ASTNode()
{
    castType = "";
}

CAST_EXPR_Node::~CAST_EXPR_Node()
{
}

void CAST_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    if ( castType == "" )
        fprintf(fout, "%s<Cast_expr>\n", tab.c_str());
    else
        fprintf(fout, "%s<Cast_expr type=\"%s\">\n", tab.c_str(), castType.c_str());

    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Cast_expr>\n", tab.c_str());

}

void CAST_EXPR_Node::setCastType(string _castType)
{
    castType = _castType;
}

string CAST_EXPR_Node::getCastType()
{
    return castType;
}


///////////////////////////////////////////////////////////
// Unary expression Node Implementation
///////////////////////////////////////////////////////////
UNARY_EXPR_Node::UNARY_EXPR_Node():ASTNode()
{
    production="";
}

UNARY_EXPR_Node::~UNARY_EXPR_Node()
{
}

void UNARY_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Unary_expr>\n", tab.c_str());
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Unary_expr>\n", tab.c_str());


}

void UNARY_EXPR_Node::setProduction(char *i)
{
    production = i;
}

void UNARY_EXPR_Node::setOperator(char *i)
{
    opt = i;
}

///////////////////////////////////////////////////////////
// Argument expression list Node Implementation
///////////////////////////////////////////////////////////
POSTFIX_EXPR_Node::POSTFIX_EXPR_Node():ASTNode()
{
    production="";
}

POSTFIX_EXPR_Node::~POSTFIX_EXPR_Node()
{
}

void POSTFIX_EXPR_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Postfix_expr>\n", tab.c_str());
    if (production.size() != 0)
    {
        fprintf(fout, "%s    <Operator type=\"%s\" />\n", tab.c_str(), production.c_str());
    }
    // Increase tab count for my children
    ++tabCount;
    // Output my children, will call corresponding dumpout because of polymorphism
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;
    fprintf(fout, "%s</Postfix_expr>\n", tab.c_str());


}

void POSTFIX_EXPR_Node::setProduction(char *i)
{
    production = i;
}



///////////////////////////////////////////////////////////
// String Node Implementation
///////////////////////////////////////////////////////////
STRING_Node::STRING_Node():ASTNode()
{
}

STRING_Node::~STRING_Node()
{
}

void STRING_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();

    fprintf(fout, "%s<String value=\"%s\" />\n", 
            tab.c_str(), value.c_str());
}



///////////////////////////////////////////////////////////
// Identifier expression list Node Implementation
///////////////////////////////////////////////////////////
IDENTIFIER_Node::IDENTIFIER_Node():ASTNode()
{
    line=0;
    size=0;
    scope="local";
}

IDENTIFIER_Node::~IDENTIFIER_Node()
{
}

int IDENTIFIER_Node::getSize()
{
    return size;
}

void IDENTIFIER_Node::setSize(int _size)
{
    size=_size;
}

string IDENTIFIER_Node::getScope()
{
    return scope;
}

void IDENTIFIER_Node::setScope(string _scope)
{
    scope=_scope;
}

void IDENTIFIER_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();
    fprintf(fout, "%s<Identifier name=\"%s\" type=\"%s\" value=\"%s\" scope=\"%s\" size=\"%d\"/>\n", 
            tab.c_str(), name.c_str(), type_specifier.c_str(), value.c_str(), scope.c_str(), size);
}

///////////////////////////////////////////////////////////
// Constant expression list Node Implementation
///////////////////////////////////////////////////////////
CONSTANT_Node::CONSTANT_Node():ASTNode()
{
}

CONSTANT_Node::~CONSTANT_Node()
{
}

void CONSTANT_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();

    fprintf(fout, "%s<Constant type=\"%s\" value=\"%s\" />\n", 
            tab.c_str(), type_specifier.c_str(), value.c_str());
}

///////////////////////////////////////////////////////////
// Return expression list Node Implementation
///////////////////////////////////////////////////////////
RETURN_Node::RETURN_Node():ASTNode()
{
}

RETURN_Node::~RETURN_Node()
{
}

void RETURN_Node::DumpOut(FILE *fout)
{
    string tab = setIndentTab();

    fprintf(fout, "%s<Return type=\"%s\" value=\"%s\">\n", 
            tab.c_str(), type_specifier.c_str(), value.c_str());
    ++tabCount;
    for ( int i=0; i < children.size(); ++i )
        children[i]->DumpOut(fout);
    --tabCount;

    fprintf(fout, "%s</Return>\n", tab.c_str());
}
