
/************************************************************************/
/*                                                                      */
/* Juan Quiroz and Dat Ta                                               */
/* CS 660 - Compiler Construction                                       */
/*                                                                      */
/* Main driver:
   Initializes global variables, parses the command line, initializes
   file pointers, calls yyparse, and closes file pointers.
*/
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "derivednodes.h"
#include "archivereg.h"
#include "regalloc.h"

#define DEBUGLEVELS 5

enum {DBGLV0, DBGLV1, DBGLV2, DBGLV3, DBGLV4};

#define INSERTMODE 0
#define LOOKUPMODE 1

#define OVERFLOW 60

/********************
 * On the current code the source code is specified with
 * the -c source_code option.
**********************/

/*********************************
  Function Declarations
*********************************/
void Usage();
void closeFilePointers(int &dbg);

int yyparse();
/*********************************/

/*********************************
 Global variables
*********************************/
int scannerDebugLevel=1;
int parserDebugLevel=1;
int stDebugLevel=2;
int warningsDebugLevel=1;

FILE *fp=NULL;
FILE *sfout=NULL;
FILE *pfout=NULL;
FILE *fout=NULL;
FILE *fst=NULL;
FILE *f_code=NULL;

// AST file handle
FILE *f_ast=NULL;

extern FILE *yyin;

int column=1;
int lineCounter=1;
int errors=0;
int warnings=0;

int INTOVERFLOW = OVERFLOW;
int FLOATOVERFLOW = OVERFLOW;
int MAXVARNAMESIZE = OVERFLOW;

//vector <string>varTypes;

char lineBuffer[1024];
int STMode = LOOKUPMODE;

Stack *st;
Stack *codegen_st=NULL;

RegisterAllocator *regAlloc;

/*********************************/
ASTNode *astroot=NULL;

/*********************************
// Driver
*********************************/
int main(int argc, char *argv[])
{
    int c=0;
    char *outputFile=NULL;
    char *sourceCode=NULL;
    // ST dump file name
    char *stDump=NULL;

    // AST dump file name
    char *astDump=NULL;
    // ASM Code dump file name
    string asmCode="c.s";

    regAlloc = new RegisterAllocator();

    int dbg=0;

    //Init global variables
    yyin = NULL;
    st = new Stack();
    codegen_st = new Stack();

    // Check if there's any input to the compiler
    if ( argc <= 1 )
    {
        printf("No input files!\n");
        return 0;
    }

    // Parse the command line
    //while ( (c = getopt(argc, argv, "d:o:hc:s:pty")) != EOF )
    while ( (c = getopt(argc, argv, "o:hc:s:p:t:y:wdf:i:j:a:")) != EOF )
    {
        switch (c) 
        {
            case 'w':
                warningsDebugLevel = DBGLV0;
                break;

            case 'j':
                MAXVARNAMESIZE = atoi(optarg);
                if ( MAXVARNAMESIZE < 1 )
                    MAXVARNAMESIZE = 2;
                printf("Defaulting variable name size to %d.\n", MAXVARNAMESIZE);
                break;

            case 'i':
                INTOVERFLOW = atoi(optarg);
                if ( INTOVERFLOW < 1 )
                    INTOVERFLOW = 2;
                printf("Defaulting integer overflow to %d.\n", INTOVERFLOW);
                break;

            case 'f':
                FLOATOVERFLOW = atoi(optarg);
                if ( FLOATOVERFLOW < 1 )
                    FLOATOVERFLOW = 2;
                printf("Defaulting float overflow to %d.\n", FLOATOVERFLOW);
                break;

            case 'd':
                dbg = 1;
                break;

            case 't':
                //scanner debug level
                // 0 no output
                // 1 all output
                // 2 reserved words
                // 3 identifiers
                // 4 output to own file
                scannerDebugLevel = atoi(optarg);
                if ( scannerDebugLevel < 0 )
                    scannerDebugLevel = DBGLV0;
                break;

            case 'p':
                //parser debug level
                // 0 no output
                // 1 all output
                // 2 output to own file
                parserDebugLevel = atoi(optarg);
                if ( parserDebugLevel < 0 )
                    parserDebugLevel = DBGLV0;
                break;

            case 'y':
                //parser debug level
                // 0 no output
                // 1 all output
                // 2 all output with var values
                stDebugLevel = atoi(optarg);
                if ( stDebugLevel < 0 )
                    stDebugLevel = DBGLV0;
                break;


            case 'a':
                // AST Dump Output file name
                outputFile = new char[strlen(optarg)+1];
                strcpy(outputFile, optarg);
                break;

            case 'o':
                // Output file name
                astDump = new char[strlen(optarg)+1];
                strcpy(astDump, optarg);
                break;

            case 'c':
                // source code file name
                sourceCode = new char[strlen(optarg)+1];
                strcpy(sourceCode, optarg);
                break;
                
            case 's':
                //symbol table dump file
                stDump = new char[strlen(optarg)+1];
                strcpy(stDump, optarg);
                break;

            case 'h':
                // help flag 
                Usage();
                exit(1);
                break;

            default:
                // If erroneous input then display usage of compiler
                Usage();
                exit(1);
        }
    }

    // if source code not provided
    if ( sourceCode == NULL )
    {
        printf("No input source code file!!\n");
        Usage();
        exit(1);
    }

    // if output file not provided
    if ( outputFile == NULL )
    {
        printf("Writing output to file \"ofile\".\n");
        outputFile = new char[6];
        strcpy(outputFile,"ofile");
    }

    // if ST output file not provided
    if ( stDump == NULL )
    {
        printf("Writing symbol table output to file \"stdump\".\n");
        stDump = new char[7];
        strcpy(stDump,"stdump");
    }

    // if AST output file not provided
    if ( astDump == NULL )
    {
        printf("Writing AST output to file \"astdump.xml\".\n");
        astDump = new char[8];
        strcpy(astDump,"astdump.xml");
    }
    //printf("Writing ASM output to file \"asmdump.asm\".\n");
    printf("Writing ASM output to file \"c.s\".\n");

    printf("\n");

    /////////////////////////////////////////////////////
    // Open file handles
    /////////////////////////////////////////////////////
    // file pointers for source code
    yyin = fopen(sourceCode, "r");
    // if source code file is not valid
    if ( yyin == NULL )
    {
        printf("Error: source code file does not exist.\n");
        exit(1);
    }
    fp = fopen(sourceCode, "r");
    fst = fopen(stDump, "w");
    f_ast = fopen(astDump, "w");
    f_code = fopen(asmCode.c_str(), "w");

    pfout = fopen(outputFile, "w");
    if ( dbg )
    {
        sfout = fopen("oTokens", "w");
    }
    else
        sfout = pfout;
    /////////////////////////////////////////////////////



    fgets(lineBuffer, 1024, fp);
    // Source code lines are preceeded by ##
    // print line 1
    fprintf(pfout, "## %s", lineBuffer);

    /*
    varTypes.push_back("void");
    varTypes.push_back("char");
    varTypes.push_back("short");
    varTypes.push_back("int");
    varTypes.push_back("long");
    varTypes.push_back("float");
    varTypes.push_back("double");
    varTypes.push_back("signed");
    */

    // Parse source code
    yyparse();
    if ( astroot != NULL )
    {
        fprintf(f_ast, "<?xml version=\"1.0\" ?>\n");
        astroot->DumpOut(f_ast);

        astroot->Code_Gen(f_code);
    }

    {
        ofstream ofs("pickle_ast");
        boost::archive::text_oarchive oa(ofs);
        //oa.registerar.register_type(static_cast<bus_stop_corner *>(NULL))
        register_all(oa);
        oa << BOOST_SERIALIZATION_NVP(astroot);
    }

    /*
    ASTNode *readin = new ASTNode();
    {
        ifstream ifs("pickle_ast");
        boost::archive::text_iarchive ia(ifs);
        register_all(ia);
        ia >> BOOST_SERIALIZATION_NVP(readin);
        FILE *reprint = fopen("reprint", "w");
        readin->DumpOut(reprint);
        fclose(reprint);
    }
    */

    rbtree *tree = NULL;
    node **_nodes = NULL;

    // Clear globals scope tree
    while ( !st->empty() )
    {
        tree = st->pop();
        // if we want to output warnings
        if ( warningsDebugLevel == DBGLV1 )
        {
            // Get a list of nodes from the tree
            _nodes = tree->foreach();
            // foreach node
            for ( int i=0; i < tree->getSize(); ++i )
            {
                // if the variable is not used, then output a warning
                if ( _nodes[i]->getCheck() == 0 && _nodes[i]->getType() != _FUNCTION )
                {
                    printf("Line %d: Warning: unused variable '%s'\n", 
                            _nodes[i]->getLine(), _nodes[i]->getName().c_str());
                    ++warnings;
                }
            }
        }
        st->dump(fst);
        tree->clear();
    }

    // Clear globals scope tree
    // There is a bug on codegen_st
    // There's something wrong with the stack that causes the program
    // to segfault, so for now I'll leave the memory leak

    // close file pointers
    closeFilePointers(dbg);

    printf("Errors: %d - Warnings: %d\n", errors, warnings);

    delete astroot;

    return 0;
}

void Usage()
{
    printf("\nRunning the compiler:\n\
            compiler [OPTIONS] -c sourceCode\n\
            OPTIONS\n\
            -o scanner_parser_output_file\n\
                \tSpecifies the output file for the parser and scanner dumps.\n\
            -s symbol_table_output_file\n\
                \tSpecifies the output file for the symbol table dumps.\n\
            -p [0-2]\t\tParser debug level.\n\
            -t [0-4]\t\tScanner debug level.\n\
            -y [0-2]\t\tSymbol table debug level.\n\
            -w \t\tDisplay all warnings.\n\
            -h \t\tDisplay help menu.\n\
            ex. compiler -o outputFile -c sourceCode\n\n");
}

void closeFilePointers(int &dbg)
{
    fclose(yyin);
    fclose(fp);
    fclose(fst);
    fclose(f_ast);
    fclose(pfout);
    if ( dbg )
        fclose(sfout);
    fclose(f_code);
}
