
/************************************************************************/
/*                                                                      */
/* Juan Quiroz and Dat Ta                                               */
/* CS 660 - Compiler Construction                                       */
/*                                                                      */
/*
   Scanner:
   Reads the source code file and breaks it up into meaningful tokens.
   The tokens are returned to the parser.  The scanner discards comments,
   keeps track of the column and the current line.
   A debug flag determines whether the tokens are outputted to a file.
   Having the symbol !!S inside a comment will force a symbol table dump.
 */
/************************************************************************/


%{
#include <stdio.h>
#include <string.h>

#include "stack.h"
#include "scandef.h"

/************************************************************************/
/* Defines                                                              */
/************************************************************************/

extern scanReturn yylval;

#define YYSTYPE scanReturn

#include "grammar.tab.hpp"

#define INSERTMODE 0
#define LOOKUPMODE 1

enum {DBGLV0, DBGLV1, DBGLV2, DBGLV3, DBGLV4};


/************************************************************************/
/* Global variables                                                     */
/************************************************************************/

extern int column;
extern int lineCounter;
extern char lineBuffer[1024];
extern int scannerDebugLevel;
// fp reads lines from source while yyin reads tokens
// fout writes to file the debug output
extern FILE *fp, *sfout, *fst;
extern int STMode;

extern int INTOVERFLOW;
extern int FLOATOVERFLOW;
extern int MAXVARNAMESIZE;

extern FILE* yyin;

int commentNest=0;
int scannerDump=0;
extern Stack *st;

extern int errors;
extern int warnings;

inline void newLineProc()
{
    ++lineCounter; 
    column = 1; 

    fgets(lineBuffer, 1024, fp);
    fprintf(sfout, "## %s", lineBuffer);
}

//inline void common(char *tok)
inline void identifiers()
{
    column += yyleng;
    if ( scannerDebugLevel == DBGLV1 || scannerDebugLevel == DBGLV3 )
        fprintf(sfout, "Token:%s\n", yytext);
}

inline void reservedWords()
{
    column += yyleng;
    if ( scannerDebugLevel == DBGLV1 || scannerDebugLevel == DBGLV2 )
        fprintf(sfout, "Token:%s\n", yytext);
}

inline void commonWithPrint()
{
    column += yyleng;
    if ( scannerDebugLevel == DBGLV1 )
        fprintf(sfout, "Token:%s\n", yytext);
}

inline void commonWithoutPrint()
{
    column += yyleng;
}


%}


/************************************************************************/
/* Regular Definitions                                                  */
/************************************************************************/


delim	[ \t]
ws	{delim}+
digit	[0-9]
number	{digit}+

float   {digit}*"."{digit}+
integer	{digit}+

letter  [a-zA-Z]

variable     [a-zA-Z_][a-zA-Z_0-9]*
strLiteral      \"[^"]*\"

/*escape sequences*/
/*charConstant    \'[^']\'*/
/*charConstant    \'[^'\\]\'|"'\"[abfnrtv?'"\\]"'"*/
charConstant    \'[^']\'|"'\"[abfnrtv?'"\\]"'"


relOps          "<"|">"|"?"
logicOps        "^"|"&"|"|"|"~"|"!" 
mathOps         "*"|"+"|"-"|"/"|"%"|"="
otherOps        "."|","|":"|";"
delimiters      "("|")"|"["|"]"|"{"|"}"

returnDirectly  {relOps}|{logicOps}|{mathOps}|{otherOps}|{delimiters} 

%x      comment
%x      singleComment

%% 	


[ ]+	                { commonWithoutPrint(); }
[\t]	                { column += 5;          }

{returnDirectly}        {  commonWithPrint();  return(yytext[0]);       }

"->"                     {  commonWithPrint();  return(PTR_OP); }

"++"                    {  commonWithPrint();  return(INC_OP); }
"--"                    {  commonWithPrint();  return(DEC_OP); }

"<<"                    {  commonWithPrint();  return(LEFT_OP); }
">>"                    {  commonWithPrint();  return(RIGHT_OP); }

"<="                    { commonWithPrint();  return(LE_OP); }
">="                    { commonWithPrint();  return(GE_OP); }
"=="                    { commonWithPrint();  return(EQ_OP); }
"!="                    { commonWithPrint();  return(NE_OP); }

"&&"                    {  commonWithPrint();  return(AND_OP); }
"||"                    {  commonWithPrint();  return(OR_OP); }

"*="                    {  commonWithPrint();  return(MUL_ASSIGN); }
"/="                    {  commonWithPrint();  return(DIV_ASSIGN); }
"%="                    {  commonWithPrint();  return(MOD_ASSIGN); }
"+="                    {  commonWithPrint();  return(ADD_ASSIGN); }
"-="                    {  commonWithPrint();  return(SUB_ASSIGN); }

"<<="                   {  commonWithPrint();  return(LEFT_ASSIGN); }
">>="                   {  commonWithPrint();  return(RIGHT_ASSIGN); }
"&="                    {  commonWithPrint();  return(AND_ASSIGN); }
"^="                    {  commonWithPrint();  return(XOR_ASSIGN); }
"|="                    {  commonWithPrint();  return(OR_ASSIGN); }

"..."                   {  commonWithPrint();  return(ELIPSIS); }


"auto"                 { reservedWords();  return(AUTO);       }
"break"                { reservedWords();  return(BREAK);       }
"case"                 { reservedWords();  return(CASE);       }
"char"                 { reservedWords();  return(CHAR);       }
"const"                { reservedWords();  return(CONST);       }
"continue"             { reservedWords();  return(CONTINUE);       }
"default"              { reservedWords();  return(DEFAULT);       }
"do"                   { reservedWords();  return(DO);       }
"double"               { reservedWords();  return(DOUBLE);       }
"else"                 { reservedWords();  return(ELSE);       }
"enum"                 { reservedWords();  return(ENUM);       }
"extern"               { reservedWords();  return(EXTERN);       }
"float"                { reservedWords();  return(FLOAT);       }
"for"                  { reservedWords();  return(FOR);       }
"goto"                 { reservedWords();  return(GOTO);       }
"if"                   { reservedWords();  return(IF);       }
"int"                  { reservedWords();  return(INT);       }
"long"                 { reservedWords();  return(LONG);       }
"register"             { reservedWords();  return(REGISTER);       }
"return"               { reservedWords();  return(RETURN);       }
"short"                { reservedWords();  return(SHORT);       }
"signed"               { reservedWords();  return(SIGNED);       }
"sizeof"               { reservedWords();  return(SIZEOF);       }
"static"               { reservedWords();  return(STATIC);       }
"struct"               { reservedWords();  return(STRUCT);       }
"switch"               { reservedWords();  return(SWITCH);       }
"typedef"              { reservedWords();  return(TYPEDEF);       }
"union"                { reservedWords();  return(UNION);       }
"unsigned"             { reservedWords();  return(UNSIGNED);       }
"void"                 { reservedWords();  return(VOID);       }
"volatile"             { reservedWords();  return(VOLATILE);       }
"while"                { reservedWords();  return(WHILE);       }

{strLiteral}           {  
                            commonWithPrint();  
                            yylval._string =  yytext;
                            return(STRING_LITERAL);  
                       }
{variable}             {  
                            if ( yyleng > MAXVARNAMESIZE )
                            {
                                printf("Line %d: Warning: Variable name %s is too long!\n", lineCounter);
                                ++warnings;
                            }
                            identifiers();
                            yylval._string =  yytext;

                            /*
                            if ( STMode == INSERTMODE )
                                ;
                            else
                                ;
                             */

                            return(IDENTIFIER);  
                       }

{charConstant}          {  
                            commonWithPrint();  
                            yylval._char = yytext[1];
                            return(CHARACTER_CONSTANT);  
                        }
{float}                 {  
                            if ( yyleng > FLOATOVERFLOW )
                            {
                                printf("Line %d: Warning: Floating point overflow!\n", lineCounter);
                                ++warnings;
                            }
                            commonWithPrint();
                            // convert string to float
                            yylval._float = atof(yytext);
                            return(FLOATING_CONSTANT);
                        }
{integer}               {  
                            if ( yyleng > INTOVERFLOW )
                            {
                                printf("Line %d: Warning: Integer overflow!\n", lineCounter);
                                ++warnings;
                            }
                            commonWithPrint();

                            // convert string to int
                            yylval._int = atoi(yytext);
                            return(INTEGER_CONSTANT);  
                        }

"/*"                    {   
                            ++commentNest; 
                            BEGIN(comment);  
                            commonWithPrint();
                        }
<comment>"*/"           {  
                            --commentNest;
                            if ( commentNest == 0 )
                                BEGIN(INITIAL);  
                            //else
                             //   printf("nested end!");
                            commonWithPrint();
                        }
<comment>"/*"           {  
                            ++commentNest;
                            //printf("nested beg!");
                            commonWithPrint();
                        }
<INITIAL,comment>"!!S"  {
                            scannerDump = 1;
                            st->dump(fst);
                            commonWithPrint();
                        }
<comment>.              { commonWithoutPrint(); }

<INITIAL,comment>\n     { 
                            newLineProc();
                        }


"//"                    { BEGIN(singleComment); }
<singleComment>"!!S"    {
                            scannerDump = 1;
                            st->dump(fst);
                            commonWithPrint();
                        }
<singleComment>.        { /* No action and no return. */ }
<singleComment>\n       {
                            BEGIN(INITIAL);
                            newLineProc();
                        }

.                       { 
                            fflush(stdout);

                            printf("line %d: Syntax error - unexpected character.\n", lineCounter);
                            printf("%s", lineBuffer);
                            printf("%*s - unexpected character.\n", column, "^");
                           // printf("%*s - unexpected character.\n", column, "^", column);
                            ++errors;
                            //return(ERROR_TOK); 
                        }

%%
